package com.ruoyi.sim.service.impl;

import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.sim.config.SimConfig;
import com.ruoyi.sim.constant.CommConst;
import com.ruoyi.sim.domain.vo.SimSocketVo;
import com.ruoyi.sim.domain.vo.SocketWrapValue;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.net.Socket;
import java.util.HashMap;

import static com.ruoyi.sim.constant.CommConst.SOCKET_TIME_OUT;

/**
 * 1.所有Socket常开。
 * todo: connectedTimeMillis 时间。
 */
@Service
public class SocketService {

    private static final Logger l = LoggerFactory.getLogger(SocketService.class);

    private static final int INIT_SIZE = 16;
    /**
     * 6 hours
     */
    private static final long LIMIT = 1000L * 60 * 60 * 6;

    /**
     * key: ip:port
     * value:
     */
    private static HashMap<String, SocketWrapValue> cachedMap = new HashMap<>(INIT_SIZE);

    @Autowired
    private SimConfig config;
    @Autowired
    private FailedCountService failedCountService;

    /**
     * @param ip   ip v4.
     * @param port
     * @return true:socket ok!
     */
    public boolean isOk(final String ip, final Integer port) {
        if (cachedMap.containsKey(ip)) {
            Socket s = cachedMap.get(buildKey(ip, port)).getSocket();
            if (s != null) {
                return (s.isConnected() && s.isBound() && !s.isClosed());
            }
        }
        return false;
    }

    /**
     * @param ip   ip v4.
     * @param port
     * @return
     */
    public String buildKey(final String ip, final Integer port) {
        if (StringUtils.isBlank(ip)) {
            throw new IllegalArgumentException("ip error");
        }
        if (port == null || port <= CommConst.PORT_MIN || port >= CommConst.PORT_MAX) {
            throw new IllegalArgumentException("port error");
        }
        return ip + ":" + port;
    }

    /**
     * @param ssv
     * @return
     */
    public AjaxResult openOne(final SimSocketVo ssv) {
        // check.
        if (!config.isCommGlobal()) {
            l.warn("isCommGlobal == {} [模拟器通信被禁用！]", config.isCommGlobal());
            return AjaxResult.error("模拟器通信被禁用！");
        }
        //
        try {
            final String ip = ssv.getIp();
            final Integer port = ssv.getPort();
            final String key = buildKey(ip, port);
            if (!isOk(ip, port)) {
                l.info("openSocket cachedSocket is not ok!new socket ip = {}!", ip);
                closeOne(ssv);
                Socket s = new Socket(ip, port);
                s.setSoTimeout(SOCKET_TIME_OUT);
                SocketWrapValue value = new SocketWrapValue(ip, port, s, System.currentTimeMillis());
                cachedMap.put(key, value);
                // failed count reset.
                failedCountService.reset0(key);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {

        }
        return AjaxResult.success("openOneSocket Success！");
    }

    /**
     * todo:部分返回Aj结果。
     *
     * @param ssvs
     * @return
     */
    public AjaxResult openAll(final SimSocketVo[] ssvs) {
        if (!config.isCommGlobal()) {
            l.warn("isCommGlobal == {} [模拟器通信被禁用！]", config.isCommGlobal());
            return AjaxResult.error("模拟器通信被禁用！");
        }
        for (SimSocketVo ssv : ssvs) {
            openOne(ssv);
        }
        return AjaxResult.success("openAllSocket Success！");
    }

    public AjaxResult closeOne(final SimSocketVo ssv) {
        String msgOk = "关闭Socket成功！";
        if (!config.isCommGlobal()) {
            l.warn("isCommGlobal == {} [模拟器通信被禁用！]", config.isCommGlobal());
            return AjaxResult.error("模拟器通信被禁用！");
        }
        final String key = buildKey(ssv.getIp(), ssv.getPort());
        if (!cachedMap.containsKey(key)) {
            cachedMap.remove(key);
            return AjaxResult.success(msgOk);
        } else {
            try {
                Socket s = cachedMap.get(key).getSocket();
                s.getInputStream().close();
                s.getOutputStream().close();
                s.close();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                cachedMap.remove(key);
                // failed count reset.
                failedCountService.reset0(key);
                return AjaxResult.success(msgOk);
            }
        }
    }

    public AjaxResult closeAll(final SimSocketVo[] ssvs) {
        String msgOk = "关闭所有Socket成功！";
        for (SimSocketVo ssv : ssvs) {
            AjaxResult ar = closeOne(ssv);
            if (ar != null && !ar.isSuccess()) {
                return ar;
            }
        }
        return AjaxResult.success(msgOk);
    }

    /**
     * 初始化。
     */
    public void resetAll() {

    }
}
